"""
Copyright (c) 2009 [name + "(at)student.hpi.uni-potsdam.de" for name in ["murat.knecht", "franziska.haeger", "christian.schwarz"]]

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
"""
import pygame
import cfg

from drawables import Drawable
from resourcemanager import ResourceManager
from vector2D import Vector2D

class _Message():
	
	def __init__(self, msg):
		object.__init__(self)
		self.text = msg

class _SystemMessage(_Message):
	
	def __init__(self, msg):
		_Message.__init__(self, msg)
		
		self.age = 3000

		self.font_color = (220,220,220,0)

class _UserMessage(_Message):

	def __init__(self, msg):
		_Message.__init__(self, msg)
		
		self.age = 2000
		
		self.font_color = (220,220,220,0)


class __ChatBox(Drawable):
	
	def __init__(self):
		Drawable.__init__(self)
	
		self.size = Vector2D(200,180)
		self.surface = pygame.Surface(self.size.to_tuple(), cfg.SURFACEFLAGS)
		
		# Set the background-colors for the window, depending if the mouse is on it or not.
		self.bg_color_inactive = (120,120,120,200)
		self.bg_color_focus = (120,120,120,225)
		self.bg_color = self.bg_color_inactive # default is not
		
		self.font_size = 12
		self.font_gap = 2
		self.font = pygame.font.SysFont("DejaVu Sans, Arial, Helvetica, ", self.font_size) # ResourceManager.get_font(self.font_size)

		self.messages = []
		self.max_messages = int(self.size.y / (self.font_size + self.font_gap))
		
		# Gap to the surface borders.
		self.upper_border = 3
		self.lower_border = 8 # leave space for the triangle
		self.left_border = self.right_border = 5
		
		# Precalculate some numbers.
		self.content_width = self.size.x - self.left_border - self.right_border
		self.content_height = self.size.y - self.upper_border - self.lower_border
		
		# Mouse does not hover the window in the beginning.
		# If it does, change the background-color.
		self.hovering = False

			
	@property
	def image(self):
		return self.surface
	
	def hovering_mouse(self, hovering):
		"""Enable/Disable the mouse hovering mode."""
		if hovering:
			self.bg_color = self.bg_color_focus
		else:
			self.bg_color = self.bg_color_inactive

	def post_system_message(self, msg):
		"""Add a message from the game.
		
		The message is put into the queue.
		
		@param msg The text message.
		"""
		self.messages.append(_SystemMessage(msg))

	def post_user_message(self, msg):
		"""Add a message from another user in multiplayer.
		
		The message is put into the queue.
		
		@param msg The text message.
		"""
		self.messages.append(_UserMessage(msg))
	
	def update(self, tick):
		
		# Throw away old messages.
		self.messages = filter(lambda msg: msg.age - tick > 0, self.messages)
		
		# Update age for the remaining
		for msg in self.messages:
			msg.age -= tick
			
		# If we still have too many messages ...
		if len(self.messages) > self.max_messages:
			# ... then reduce the age of the oldest messages by half.
			for i in range(len(self.messages)-self.max_messages):
				self.messages[i].age /= 3
	
	def draw(self):
		# Draw background
		self.surface.fill(self.bg_color)
		
		# Draw as many messages as possible.
		x = self.left_border
		y = self.upper_border
		for msg in self.messages[:self.max_messages]:
			
			# Render message in its font and shorten it, if it is too long.
			rendered = self.font.render(msg.text, False, msg.font_color)
			area = rendered.get_rect()
			if area.width > self.content_width:
				area = (0,0, self.content_width, area.height)
			
			# Draw it on the chat box.
			self.surface.blit(rendered, (x,y), area)
			
			# Calculate position of the next message.
			y += self.font_size + self.font_gap
			
		# Draw the triangle, if necessary.
		if len(self.messages) > self.max_messages:
			x = self.left_border + self.content_width / 2 - 20
			y = self.upper_border + self.content_height + 2
			pygame.draw.polygon(self.surface, (255,255,255,150), [ (x,y), (x+40,y), (x+20,y+4)], 0)
			
	def wants_to_be_drawn(self):
		return bool(self.messages)
		
ChatBox = __ChatBox()