"""
Copyright (c) 2009 [name + "(at)student.hpi.uni-potsdam.de" for name in ["murat.knecht", "franziska.haeger", "christian.schwarz"]]

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
"""
from ConfigParser import SafeConfigParser
from ConfigParser import NoOptionError, NoSectionError, ParsingError
import os, sys
    
class ConfigReader(object):
    """The ConfigReader reads config files with standard python syntax."""
    
    def __init__(self, configfile=None):
        """Initiates the config reader.
        
        @param configfile Path to the config file. the config file can also be
        set by callling setConfigFile()
        """
        object.__init__(self)
        self._cp         = SafeConfigParser()
        self._configfile = configfile
        
        if self._configfile != None:
            self._read_configfile()
    
    def _read_configfile(self):
        """Reads the preconfigured config file."""
        self._cp.read(self._configfile)
    
    def _determine_section(self, parameter):
        """Determines the section for the given parameter.
        
        @param parameter Name of the parameter
        
        @return Name of the section which contains the parameter
        
        @throw NoSectionError if no section can be determined
        """
        value = None
        section = None
        for sec in self._cp.sections():
            try:    value = str(self._cp.get(sec, parameter))
            except: pass
            else:
                section = sec
        
        if value == None:
            raise NoSectionError(parameter)
        else:
            return section
    
    def set_config_file(self, configfile):
        """Sets a new config file. Config file is read automaticaly.
        
        @param configfile Path to the config file.
        
        @throw IOError if file does not exist.
        """
        self._configfile = configfile
        
        if self._configfile != None:
            if os.path.isfile(configfile):
                self._readConfigFile()
            else:
                raise IOError()
    
    def get_config_filename(self):
        """Returns the name of the corrent config file.
        
        @return String which contains the configfile name.
        """
        return str(self._configfile)
    
    def write_config(self, filename=None):
        """Writes the config back to disc.
        
        @param filename Path where the config should be stored. If it is not
        specified the config will be the one set with the last setConfigFile() call.
        """
        if self._configfile == None:
            return
        
        if filename:
            self._cp.write(file(filename, "wb"))
        else:
            self._cp.write(file(self._configfile))

    def get_parameter(self, section = None, parameter = None):
        """Returns the value for a specified configuration parameter.
        
        @param section Name of the configuration section. If the section is not
        specified the ConfigReader tries to determine the correct section.
        If the same parameter is found in multiple sections the value of the
        first section is returned.        
        @param parameter Name of the parameter which should be read.
        
        @return String representation of the configuration parameter
        
        @throw NoSectionError if no section with the parameter is found.
        @throw NoOptionError  if no parameter is specified.
        """
        if not (section or parameter):
            return None
        
        if not parameter:
            raise NoOptionError(parameter, section)
        
        if not section:
            section = self._determine_section(parameter)
        
        if not section:
            return None
        
        return eval(self._cp.get(section, parameter))
    
    def set_parameter(self, section=None, parameter=None, value=None):
        """Sets the configuration file parameter. writeConfig() has to be called
        to persist the changes.
        
        @param section Name of the configuration section. If the section is not
        specified the ConfigReader tries to determine the correct section.
        If the same parameter is found in multiple sections the value of the
        first occurence is set.
        @param parameter Name of the parameter which should be set.
        @param value New value of the parameter. The value is normalized to
        its string representation. writeConfig() has to be called to persist
        the changes.
        
        @throw NoSectionError if no section with the parameter is found. The
        section is determined automaticaly, if it is not specified.
        @throw NoOptionError if no parameter is specified.
        """
        if section == None and parameter == None:
            return
        
        if parameter == None:
            raise NoOptionError(parameter, section)
        
        if section == None:
            section = self._determine_section(parameter)
        
        try:    self._cp.get(section, parameter)
        except: raise NoSectionError(section)
        else:   self._cp.set(section, parameter, str(value))
    
    def get_section_names(self, containing = None):
        """Returns the names of all sections defined in the config file.
        
        @param containing Return only sections which contains this string.
        
        @return Array containing the names of all sections defined in the config file
        except the [DEFAULT] section.
        
        @throw ParsingError if the config file is not set.
        """
        if self._configfile == None: raise ParsingError("No config file defined")
        
        if containing == None:
            return self._cp.sections()
        else:
            res = []
            for section in self._cp.sections():
                if containing in section:
                    res.append(section)
            return res
    
    def get_parameter_names(self, section = None):
        """Returns the names of all Parameter specified in a specific section.
        
        @param section Name of the section, which parameters should be returned.
        
        @return Array containing the names of all parameters of the defined section.
        
        @throw ParsingError if the config file is not set.
        @throw NoSectionError if the specified section does not exist.
        """
        if section == None or not self._cp.has_section(section):
            raise NoSectionError(section)
        
        if self._configfile == None: raise ParsingError("No config file defined")
        
        return self._cp.options(section)