"""
Copyright (c) 2009 [name + "(at)student.hpi.uni-potsdam.de" for name in ["murat.knecht", "franziska.haeger", "christian.schwarz"]]

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
"""
import pygame

from tile import Tile
from resourcemanager import ResourceManager
from vector2D import Vector2D
import events, cfg
from weaponsmith import WeaponSmith

from windowcontent import WindowContent
from animations import LawnMower

from drawables import Drawable
class WeaponIconSprite(Drawable):
	"""Sprite for a WeaponIcon in the weapon selection menu."""
	
	def __init__(self, weapon):
		"""Initializes the icon.
		
		@param position Initial position for the tank."""
		
		Drawable.__init__(self)
		self.weapon = weapon
		
		# Store the image, so we can always retrieve the unmodified version.
		self.image = self.originalImage = weapon.icon
		
		# Initialize the fonts to use.
		self.font = ResourceManager.get_font(12)
		self.costSurface = None
		self.damageSurface = None
		self.helperPosition = None
	
	def draw(self, surface, position):
		# Recalculate the rectangle of the icon, which we can later use to determine,
		# if an icon has been clicked.
		self.rect = pygame.Rect(position[0],position[1], self.image.get_rect().width, self.image.get_rect().height)
		
		# Draw the icon
		surface.blit(self.image, position)
		
		# Draw the costs on the top right corner
		self.costSurface = self.font.render(str(self.weapon.costs), True, pygame.Color(0,0,0))
		self.helperPosition = Vector2D(position).add((self.image.get_rect().width-5, 0))
		surface.blit(self.costSurface, self.helperPosition.to_tuple())
		# Draw the costs on the top right corner
		self.damageSurface = self.font.render(str(self.weapon.damage), True, pygame.Color(255,0,0))
		self.helperPosition = Vector2D(position).add((self.image.get_rect().width-5, 0))
		self.helperPosition.add((0,self.image.get_rect().height-10))
		surface.blit(self.damageSurface, self.helperPosition.to_tuple())

class TowerMenu(object):
	def __init__(self, screen):
		self.screen = screen
		self.height = 40
		self.overlap = 24
		self.cooldown = 0
		self.maxCooldown = 0
		self.waveCnt = 0
		
		self.lawnmower = LawnMower()
		
		# Get the image for the credits ready.
		self.lifes_image = ResourceManager.get_image("heart.png")
		self.credit_image = ResourceManager.get_image("menu_credits.png")
		self.toolbarImage = ResourceManager.get_image("toolbar.png")
		self.wave_image = ResourceManager.get_image("swarm.png")
		
		# Create the surface on which to draw the menu.
		# Note, that all coordinates are relative to this surface,
		# but its contents are shared with the screen.
		self.surface = self._create_subsurface(pygame.display.Info())
		
		# Load the available tower sprites.
		self._load_towers()
		
		# Initialize the font to use.
		self.font = ResourceManager.get_font(24)
	
	@property
	def image(self):
		return self.surface

	def set_surface(self, info):
		"""Should be called, if the screensize was changed.
		
		@param info pygame.display.Info object.
		"""
		self.surface = self._create_subsurface(info)
	
	def set_credits(self, credits):
		self.credits = credits
	
	def set_lifes(self, lifes):
		self.lifes = lifes
	
	def set_wave(self, wave, cooldown):
		self.wave = wave
		if self.cooldown < cooldown:
			self.maxCooldown = cooldown
			self.waveCnt += 1
		
		self.cooldown = cooldown
	
	def update(self, tick):
		pass

	def draw(self):
		"""Draws the bottom menu."""
		
		# draw the background
		self.surface.fill(pygame.Color(0,0,0,0))
		self.surface.blit(self.toolbar, (0,0))
		
		# Distance to the left border
		x = 0
		x += self._draw_lifes(x)
		x += 10 + self._draw_credits(x)
		
		# Draw the towers.
		x += 5 # Distance to credits
		for tower in self.towers:
			# draw the icon and the cost
			tower.draw(self.surface, (x, 5 + self.overlap))
			# Move one place further to the right
			x += tower.image.get_rect().width + 15
		
		x = pygame.display.Info().current_w
		x = self._draw_wave(x) - 10
		x = self._draw_cooldown(x)
	
	def handle_mouse_event(self, event_type, position, button):
		"""Handles events for the tower menu.
		
		@param event the event to be handled, available attributes in event: pos, button
		"""
		if event_type == pygame.MOUSEBUTTONUP:
			# the user possibly requested an action
			if button == 1:
				for tower in self.towers:
					if tower.click_check(position):
						events.add_build_tower(tower.weapon.identifier)
						return
	
	def _create_subsurface(self, info):
		"""Creates a subsurface for the tower menu."""
		# Create the child surface, so we can handly coordinates
		# in an absolute fashion.
		rect = 0, info.current_h - self.height - self.overlap, info.current_w, self.height + self.overlap
		self.topleft = 0, info.current_h - self.height - self.overlap
		
		# creates an image representing the toolbar background
		self.toolbar = pygame.Surface((info.current_w, self.height + self.overlap), cfg.SURFACEFLAGS)
		for offset in xrange(0, self.toolbar.get_rect().width, self.toolbarImage.get_rect().width):
			self.toolbar.blit(self.toolbarImage, (offset, 0))
		
		return pygame.Surface((info.current_w, self.height + self.overlap), cfg.SURFACEFLAGS)

	def _draw_credits(self, x):
		# Draw the background image for the credit display.
		self.surface.blit(self.credit_image, (x, self.overlap))
		
		# Draw the amount of credits available to the player.
		amount_surface = self.font.render(str(self.credits), True, pygame.Color(0,155,0))
		shadow_surface = self.font.render(str(self.credits), True, pygame.Color(0,0,0))
		
		# @todo read size from font
		h = self.credit_image.get_rect().height / 2 - (amount_surface.get_rect().height / 2)
		h += self.overlap
		
		self.surface.blit(shadow_surface, (x+51, h+1))
		self.surface.blit(amount_surface, (x+50, h))
		
		return self.credit_image.get_rect().width

	def _draw_lifes(self, x):
		# Draw the background image for the credit display.
		self.surface.blit(self.lifes_image, (x, self.overlap))
		
		# Draw the amount of credits available to the player.
		lifes_surface = self.font.render(str(self.lifes), True, (255,255,255))
		
		# @todo read size from font
		imgx = x + self.lifes_image.get_rect().width / 2 - lifes_surface.get_rect().width / 2
		imgy = self.lifes_image.get_rect().height / 2 - lifes_surface.get_rect().height / 2
		imgy += self.overlap
		
		self.surface.blit(lifes_surface, (imgx, imgy))
		
		return x + self.lifes_image.get_rect().width

	def _draw_cooldown(self, x):
		# 164 = width of lawnmover
		x -= 164
		if self.maxCooldown:
			cooldownpercentage = self.cooldown * 100 / self.maxCooldown
		else:
			cooldownpercentage = 0
		
		self.lawnmower.update(cooldownpercentage, self.waveCnt)
		self.surface.blit(self.lawnmower.image, (x, self.overlap))		
		return x

	def _draw_wave(self, x):
		# Draw the background image for the credit display.
		self.surface.blit(self.wave_image, (x-self.wave_image.get_rect().width, self.overlap))
		
		# Draw the number of waves.
		wave_surface = self.font.render(str(self.wave), True, (255,0,0))
		
		# @todo read size from font
		imgx = x - self.wave_image.get_rect().width / 2 - wave_surface.get_rect().width / 2
		imgy = self.wave_image.get_rect().height / 2 - wave_surface.get_rect().height / 2
		imgy += self.overlap
		
		self.surface.blit(wave_surface, (imgx, imgy))
		
		return x - self.wave_image.get_rect().width

	def _load_towers(self):
		"""Initializes the towers."""
		self.towers = []
		self.towers.append(WeaponIconSprite(WeaponSmith.get_weapon("FrogTower")))
		self.towers.append(WeaponIconSprite(WeaponSmith.get_weapon("InsecticideTower")))
		self.towers.append(WeaponIconSprite(WeaponSmith.get_weapon("FlyFlapTower")))
		self.towers.append(WeaponIconSprite(WeaponSmith.get_weapon("HoneyPotTower")))
		