"""
Copyright (c) 2009 [name + "(at)student.hpi.uni-potsdam.de" for name in ["murat.knecht", "franziska.haeger", "christian.schwarz"]]

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
"""
import math
import pygame

from chat import ChatBox
from vector2D import Vector2D
from resourcemanager import ResourceManager
from tile import Tile
from weaponsmith import WeaponSmith
import cfg

from controller import Controller
class TowerController(Controller):
	def __init__(self, gamecontroller, level, enemycontroller):
		Controller.__init__(self)
		self.gamecontroller = gamecontroller
		self.enemycontroller = enemycontroller
		self.level = level
		self.surface = pygame.Surface(self.level.pxl_size, cfg.SURFACEFLAGS)
		self.placingTower = None
		self.towerGroup = pygame.sprite.Group()
		
		# Mouse does not hover over any tower yet.
		self.hovering_color = (150,0,0,128)
		self.hovering_color_darker = True
		
		# Sounds for building
		self.badIdea = ResourceManager.get_sound("badidea.wav")
		self.goodIdea = ResourceManager.get_sound("goodidea.wav")
		self.fontBlack  = ResourceManager.get_font(20)
		self.fontColor = ResourceManager.get_font(18)
		self.cost1Surface = None
		self.cost2Surface = None
		self.damage1Surface = None
		self.damage2Surface = None
	
	@property
	def image(self):
		return self.surface

	def initialize_nextLevel(self, level):
		self.level = level
		self.placingTower = None
		self.towerGroup = pygame.sprite.Group()
		
		# Mouse does not hover over any tower yet.
		self.hovering_color = (150,0,0,128)
		self.hovering_color_darker = True

	def handle_build_tower_event(self, event):
		self.placingTower = WeaponSmith.get_weapon(event.weapon)
	
	def handle_mouse_hovering(self, pos):
		if pos:
			pos = self.recalculate_position(pos)
			
			for tower in self.towerGroup:
				tower.hover = tower.rect.collidepoint(pos)
		else:
			for tower in self.towerGroup:
				tower.hover = False
	
	def tower_can_be_placed(self):
		# test collision with non buildable tile
		if pygame.sprite.spritecollide(self.placingTower, self.level.nonbuildables, False, pygame.sprite.collide_rect):
			return False
		
		# test collision with other towers
		if pygame.sprite.spritecollide(self.placingTower, self.towerGroup, False, pygame.sprite.collide_rect):
			return False
		
		return True
	
	def get_tower_for_position(self, position):
		"""Tests for a tower on the given position.
		"""		
		for tower in self.towerGroup:
			if tower.click_check(position):
				return tower
		
		return None

	def sell_tower(self, position):
		"""Sells the tower under the position."""
		tower = self.get_tower_for_position(position)		
		if not tower:
			return
		
		tower.sell_weapon()
		self.towerGroup.remove(tower)		
	
	def update_tower(self, position):
		"""Tries to update the tower on the given position."""
		tower = self.get_tower_for_position(position)		
		if not tower:
			return
		
		if not tower.update_weapon(tower.level + 1, self.gamecontroller.waste_resources):
			self.badIdea.play()
			return
		
		self.goodIdea.play()
	
	def place_tower(self, event):
		"""Places the tower, if possible.
		
		@return Returns True, if the event was handled, False otherwise.
		"""
		if not self.placingTower:
			return False
		
		if event.button == 3:
			self.placingTower.hover = False
			self.placingTower = None
			return True
		
		self.placingTower.rect.center = self.recalculate_position(event.pos)
		if not self.tower_can_be_placed() or not self.gamecontroller.waste_resources(self.placingTower.costs):
			self.badIdea.play()
			return True
		
		self.goodIdea.play()
		
		self.towerGroup.add(self.placingTower)
		self.placingTower.build()
		self.placingTower = WeaponSmith.get_weapon(self.placingTower.identifier)
		
		return True
	
	def update(self, tick):
		if self.placingTower:
			self.placingTower.rect.center = self.recalculate_position(pygame.mouse.get_pos())
		
		# Includes cooldown, so we must fire afterwards!
		self.towerGroup.update(tick)
		
		# Now that cooldowns and so on have been calculated, FIRE!
		self._determine_and_destroy_enemy()
		
		# Update the hovering color, so we get a pulsing effect.
		self._update_hovering_color(tick)

	def _determine_and_destroy_enemy(self):
		"""Gets for each tower the nearest enemy and inflicts some damag.
		
		Cooldown and range are considered.
		"""
		#filter inactive towers
		activetowers = filter(lambda tower: not tower.cooldown_left, self.towerGroup)
		
		# For each tower, fire, if possible.
		for tower in activetowers:			
			### Try, if filter is faster
			## If the tower is still cooling down, it cannot fire.
			#if tower.cooldown_left:
			#	continue
			
			# If the tower does not have a valid enemy, try to set one
			if not tower.has_valid_enemy():
				# Get nearest enemy in range of this tower
				nearest_enemy = self.enemycontroller.nearest_enemy(tower.rect.center, tower.attackables)
				tower.set_enemy(nearest_enemy)
			
			# continue, if we (still) do not have an enemy in range
			if not tower.has_valid_enemy():
				continue
			
			# Determine the enemies to damage, considering the tower's area of effect.
			enemies = []
			if not tower.aoe:
				# Inflict damage to the bug, and remove it as towers enemy, if it was killed.
				tower.attackedPos = tower.enemy.rect.center
				enemies = [ tower.enemy ]
			else:
				# We inflict damage to an entire area. So, first we must determine the tile to fire upon.
				center = tower.target_tile # Did the user select a fixed one?
				if not center:
					# If not, we determine the tile that our nearest enemy is on.
					center = self.level.pxl_to_tile_coordinates([tower.enemy.position.x, tower.enemy.position.y])
				
				# Save, where we want the attack animation to appear.
				tower.attackedPos = self.level.tile_to_pxl(center).add((Tile.size[0]/2,Tile.size[1]/2)).to_tuple()
				
				# Now we determine the area of effect, i.e. the square of tiles where all
				# enemies are hit. (x-area -> x+area, y-area -> y+area)
				area = tower.aoe-1
				tiles = [ (x,y) for x in range(center[0]-area, center[0] + area+1) for y in range(center[1]-area, center[1] + area+1) ]
				
				# Ignore tile positions that are outside the map.
				tiles = filter(lambda pos: self.level.is_valid_tile_position(pos), tiles)
				# Ignore tiles that are not moveable, where no bug can go.
				# DOES NOT WORK FOR SOME REASON :(
				#tiles = filter(lambda pos: self.level.get_tile(pos).movable, tiles)
				
				# Now get the enemies that will be hit.
				enemies = self.enemycontroller.get_enemies_on_tiles(tiles)
				tower.set_enemy()
			
			# Attack!
			total_damage = 0
			for enemy in enemies:
				# Inflict damage to all bugs on the tiles.
				damage = self.enemycontroller.hurt_enemy(enemy, tower.damage, tower.damagetype)
				if tower.hasBuffs:
					enemy.add_buffs(tower.get_buffs())
				
				total_damage += damage
				
				# Didn't do any damage? Post it.
				if 0 == damage:
					ChatBox.post_system_message("%s resisted %s's attack." % (enemy.displayname , tower.shortname))
				
				# Remove the target enemy, if we killed it.
				if 0 >= enemy.hp and enemy is tower.enemy:
					tower.set_enemy()
			
			# Post what the tower achieved.
			if 0 == total_damage:
				if 1 < len(enemies):
					ChatBox.post_system_message(tower.shortname + " did not cause any damage at all!")
			else:
				pass
				#ChatBox.post_system_message(tower.shortname + " caused " + str(total_damage) + " damage.")

			# Play sound.
			tower.sound.play()
			
			# Now, weapon must cool down, before it can fire again.
			tower.cooldown_left = tower.cooldown
	
	def draw(self):
		self.surface.fill((0,0,0,0))
		if self.placingTower:
			#self.placingTower.update_buildstate(True)
			self.placingTower.update_buildstate(self.tower_can_be_placed())
			self.surface.blit(self.placingTower.image, self.placingTower.rect.topleft)
			width = self.placingTower.range * Tile.size[0]
			pygame.draw.circle(self.surface, self.hovering_color, self.placingTower.rect.center, width, 4)

		# The Towers themselves
		self.towerGroup.draw(self.surface)
		
		for tower in self.towerGroup:
			# Attacks
			tower.draw_attack(self.surface)
			
			# Show range and Update Cost
			if tower.hover:
				width = tower.range * Tile.size[0]
				pygame.draw.circle(self.surface, self.hovering_color, tower.rect.center, width, 2)
				
				# Draw the costs for upgrading the tower to the next level, if any.
				if tower.has_upgrade():
					self.cost1Surface = self.fontBlack.render(("Up: "+str(tower.get_upgrade_costs())), True, (0,0,0,125))
					self.cost2Surface = self.fontColor.render(("Up: "+str(tower.get_upgrade_costs())), True, (255,255,0,200))
				
				# Draw the amount of damage the tower currently does.
				self.damage1Surface = self.fontBlack.render(("Dmg: "+str(tower.damage)), True, (0,0,0,125))
				self.damage2Surface = self.fontColor.render(("Dmg: "+str(tower.damage)), True, (255,0,0,200))
				
				#imgy -= 20
				#self.surface.blit(cost_surface, (imgx, imgy))
				if tower.has_upgrade():
					self.surface.blit(self.cost1Surface, (tower.rect.centerx, tower.rect.centery-40))
					self.surface.blit(self.cost2Surface, (tower.rect.centerx, tower.rect.centery-40))
				self.surface.blit(self.damage1Surface, (tower.rect.centerx, tower.rect.centery-20))
				self.surface.blit(self.damage2Surface, (tower.rect.centerx, tower.rect.centery-20))
	
	def _update_hovering_color(self, tick):
		(r,g,b,a) = self.hovering_color
		difference = tick / 10
		
		if self.hovering_color_darker:
			r += difference
			if r > 220:
				r = 220
				self.hovering_color_darker = False
		else:
			r -= difference
			if r < 150:
				r = 150
				self.hovering_color_darker = True
		
		self.hovering_color = (r,g,b,a)